<?php
  /**
 * This file is part of a project maintained at  NC State University.
 *
 * More information is included in the project README.md.
 *
 * (c) NC State University
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace NCSU\Auth;

use NCSU\Auth\Adapter\AdapterInterface;
use NCSU\Auth\Exception\RuntimeException;
use NCSU\Auth\Storage\MemoryStorage;
use NCSU\Auth\Storage\StorageInterface;

/**
 * This represents a user-friendly, use-case driven service wrapper around
 * an authentication adapter and a storage implementation.
 *
 * @author Dustin Wheeler <mdwheele@ncsu.edu>
 */
class AuthService implements AuthServiceInterface
{
    /**
     * A storage implementation.
     * @var Storage\StorageInterface
     */
    protected $storage;

    /**
     * An authentication adapter.
     * @var Adapter\AdapterInterface
     */
    protected $adapter;

    function __construct(AdapterInterface $adapter = null, StorageInterface $storage = null)
    {
        if ($adapter !== null) {
            $this->setAdapter($adapter);
        }

        if ($storage !== null) {
            $this->setStorage($storage);
        }
    }

    /**
     * @param mixed $adapter
     *
     * @return AuthService Fluent interface.
     */
    public function setAdapter(AdapterInterface $adapter)
    {
        $this->adapter = $adapter;
        return $this;
    }

    /**
     * Retrieves the current authentication adapter.
     *
     * @todo Set default adapter to Wrap or Shibboleth.
     *
     * @return Adapter\AdapterInterface|null
     */
    public function getAdapter()
    {
        return $this->adapter;
    }

    /**
     * @param Storage\StorageInterface $storage
     *
     * @return AuthService Fluent interface.
     */
    public function setStorage(StorageInterface $storage)
    {
        $this->storage = $storage;
        return $this;
    }

    /**
     * Returns the storage implementation.
     *
     * If none is provided, the default storage implementation is in-memory.
     *
     * @return Storage\StorageInterface
     */
    public function getStorage()
    {
        if ($this->storage === null) {
            $this->setStorage(new MemoryStorage());
        }

        return $this->storage;
    }

    /**
     * Attempts to authenticate a user and provides an authentication result.
     * @throws Exception\RuntimeException
     * @return AuthResult
     */
    public function authenticate()
    {
        if ($this->adapter === null) {
            throw new RuntimeException('You must provide an implementation of an authentication adapter.');
        }

        $result = $this->adapter->authenticate();
        
        // Log user out to refresh data.
        if ($this->hasIdentity()) {
            $this->logout();
        }

        if ($result->isValid()) {
            $identity = $result->getIdentity();
            $this->getStorage()->write($identity);
        }

        return $result;
    }

    /**
     * Returns true if identity exists. False otherwise.
     * @return boolean
     */
    public function hasIdentity()
    {
        return $this->getStorage()->isEmpty() === false;
    }

    /**
     * Retrieves the identity if it exists. Null otherwise.
     * @return mixed|null
     */
    public function getIdentity()
    {
        $storage = $this->getStorage();

        if ($storage->isEmpty()) {
            return null;
        }

        return $storage->read();
    }

    /**
     * Clears the authenticated identity's session.
     * @return void
     */
    public function logout()
    {
        $this->getStorage()->clear();
    }

    /**
     * Returns true if the user is authenticated, false otherwise
     * @return bool
     */
    public function isAuthenticated()
    {
        return $this->adapter->isAuthenticated();
    }
}